package gssim.db.mysql;

import gssim.db.dao.data.Plugin;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

public class PluginDAO extends AbstractDAO implements
		gssim.db.dao.PluginDAO {

	protected PluginDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(PluginDAO.class);
		setRelationName("plugin");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM plugin WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {

		String sql = "SELECT * FROM plugin WHERE (id = ? OR ISNULL(?))"
				+ " AND (name = ? OR ISNULL(?)) AND (is_global = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO plugin(name, is_global, description, input"
				+ ") VALUES(?, ?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM plugin WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE plugin SET name = ?, is_global = ?, "
				+ "description = ?, input = ? WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(Plugin obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(Plugin obj, ResultSet resultSet)
			throws SQLException {

		ResultSetAdapter result = (ResultSetAdapter) resultSet;

		Integer pK = result.getIntObj("id");
		String name = result.getString("name");
		String description = result.getString("description");
		String input = result.getString("input");
		Boolean isGlobal = result.getBooleanObj("is_global");

		obj.setPK(pK);
		obj.setName(name);
		obj.setDescription(description);
		obj.setInput(input);
		obj.setIsGlobal(isGlobal);
	}

	@Override
	public int fillStatementWithObject(PreparedStatement pstmt, Plugin obj,
			int offset) throws SQLException {

		PreparedStatementAdapter stmt = (PreparedStatementAdapter) pstmt;

		stmt.setString(offset + 1, obj.getName());
		stmt.setBoolean(offset + 2, obj.getIsGlobal());
		stmt.setString(offset + 3, obj.getDescription());
		stmt.setString(offset + 4, obj.getInput());

		return 4;

	}

	@Override
	public List<Plugin> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<Plugin> list = new ArrayList<Plugin>();

		stmt.setInt(1, finder.getInteger("id"));
		stmt.setInt(2, finder.getInteger("id"));
		stmt.setString(3, finder.getString("name"));
		stmt.setString(4, finder.getString("name"));
		stmt.setBoolean(5, finder.getBoolean("global"));
		stmt.setBoolean(6, finder.getBoolean("global"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			Plugin obj = new Plugin();
			fillObjectFromResultSet(obj, result);
			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(Plugin obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		fillStatementWithObject(stmt, obj, 0);

		int result = stmt.executeUpdate();

		if (result == 1) {
			ResultSet gk = stmt.getGeneratedKeys();
			Integer pK = gk.next() ? gk.getInt(1) : -1;
			obj.setPK(pK);

			if (commit)
				commit();

			logSuccess("insert");

		} else
			logError("insert", result, obj);

		return result;
	}

	public int[] insertBatch(List<Plugin> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			fillStatementWithObject(stmt, objList.get(i), 0);
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {

			ResultSet gk = stmt.getGeneratedKeys();
			for (int i = 0; i < objList.size(); i++) {
				Integer PK = gk.next() ? gk.getInt(1) : -1;
				objList.get(i).setPK(PK);
			}

			logSuccess("batch insert", j);

			if (commit)
				commit();

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public Plugin select(Plugin obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getPK());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();

		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			fillObjectFromResultSet(obj, result);
			return obj;
		}

		// no user found with specified id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(Plugin obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();

		int offset = fillStatementWithObject(stmt, obj, 0);

		stmt.setInt(offset + 1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
